#!/usr/bin/python
# -*- coding: latin-1 -*-
"""
cloudGui
The gui window with widgets for setting cloud automation parameter values.

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
@requires: wxWidgets for Python
"""

import wx
import util
import cloudParameters
import copy

ColourGroup1 = (210,210,210)
ColourGroup2 = (210,235,210)
ColourGroup3 = (225,200,200)
ColourGroup4 = (210,210,235)
ColourGroup5 = (200,225,200)
ColourGroup6 = (235,210,210)
ColourGroup7 = (200,200,225)
ColourGroup8 = (235,235,200)
ColourGroup9 = (200,220,220)
ColourGroup10 = ColourGroup7#(220,220,200)
ColourGroup11 = (200,235,235)
ColourGroup12 = (220,200,220)
ColourGroup13 = ColourGroup8

class CloudGUI(wx.Frame):
    """
    The GUI window for setting automation parameter values.
    """

    def __init__(self, parent, eventCaller):
        """
        Class constructor.
        
        @param self: The object pointer.
        @param parent: The window parent.
        """

        # call the parent classes constructor to begin with 
        wx.Frame.__init__(self, parent, -1, title='Cloud Preset Controls', pos=(255,3))
        
        self.Bind(wx.EVT_CLOSE, self.onClose)

        self.panel = wx.Panel(self, -1)

        self.eventCaller = eventCaller
        self.isEdited = False
        
        # make a Sizer to hold widgets
        siz1vgap = 3
        siz1hgap = 8
        self.sizer1 = wx.GridBagSizer(siz1vgap, siz1hgap)
        self.sizer1.SetEmptyCellSize((0,0))
        # add labels to left and top
        self.addLabels()
        
        # The following specifies the names of all widgets.
        # Methods for automatic generation and layout of widgets build the GUI from this list of widget names.
        # The widget base names are expanded with 'Start', 'Time1', 'Curve1', 'Middle', 'Time2', 'Curve2' and 'End'
        # wx Text controls are generated, and the variable name will be expanded with 'Ctrl', e.g. self.v1AmpStartCtrl
        
        # widget names
        self.widgetBasenames = copy.copy(cloudParameters.parameterBasenames)
        self.widgetFullnames = [] 
        self.populateWidgetFullnames()
        # set widget IDs
        self.setWidgetIDs(self.widgetFullnames)
        # make gui widgets and put in sizer for auto layout
        for name in self.widgetFullnames:
            self.makeTextCtrlFromWidgetName(name)
            self.putWidgetInSizer(name)

        # finish the definition of the sizer
        # bottom spacer
        self.sizer1.Add((0, 20), (36,17))
        
        # set the size of the window1
        # to fit the space needed for the sizer
        self.sizer1.SetSizeHints(self)
        self.SetSizer(self.sizer1)

    def onClose(self, event):
        if not wx.CloseEvent.CanVeto(event):
            print 'destroy module window'
            self.Destroy()
        else:
            print 'do not destroy module window'
            wx.CloseEvent.Veto(event)
            self.Show(False)
        print 'subwindow closed'

    def addLabels(self):
        # labels
        self.amp1Label = wx.StaticText(self.panel, -1, "Amp1", (0, 0))
        self.amp2Label = wx.StaticText(self.panel, -1, "Amp2", (0, 0))
        self.amp3Label = wx.StaticText(self.panel, -1, "Amp3", (0, 0))
        self.amp4Label = wx.StaticText(self.panel, -1, "Amp4", (0, 0))
        self.grRt1Label = wx.StaticText(self.panel, -1, "GrRt1", (0, 0))
        self.grRt2Label = wx.StaticText(self.panel, -1, "GrRt2", (0, 0))
        self.grRt3Label = wx.StaticText(self.panel, -1, "GrRt3", (0, 0))
        self.grRt4Label = wx.StaticText(self.panel, -1, "GrRt4", (0, 0))
        self.syncLabel = wx.StaticText(self.panel, -1, "sync", (0, 0))
        self.trsp1Label = wx.StaticText(self.panel, -1, "Trsp1", (0, 0))
        self.trsp2Label = wx.StaticText(self.panel, -1, "Trsp2", (0, 0))
        self.trsp3Label = wx.StaticText(self.panel, -1, "Trsp3", (0, 0))
        self.trsp4Label = wx.StaticText(self.panel, -1, "Trsp4", (0, 0))
        self.dur1Label = wx.StaticText(self.panel, -1, "Dur1", (0, 0))
        self.dur2Label = wx.StaticText(self.panel, -1, "Dur2", (0, 0))
        self.dur3Label = wx.StaticText(self.panel, -1, "Dur3", (0, 0))
        self.dur4Label = wx.StaticText(self.panel, -1, "Dur4", (0, 0))
        self.sweepToLabel = wx.StaticText(self.panel, -1, "sweepTo", (0, 0))
        self.sweepTimeLabel = wx.StaticText(self.panel, -1, "SweepShape", (0, 0))
        self.atckLabel = wx.StaticText(self.panel, -1, "Atck", (0, 0))
        self.shapeLabel = wx.StaticText(self.panel, -1, "Shape", (0, 0))
        self.gDistLabel = wx.StaticText(self.panel, -1, "gDist", (0, 0))
        self.fmFqLabel = wx.StaticText(self.panel, -1, "FmFq", (0, 0))
        self.fmIdxLabel = wx.StaticText(self.panel, -1, "FmIdx", (0, 0))
        self.nFmFqLabel = wx.StaticText(self.panel, -1, "nFmFq", (0, 0))
        self.nFmIdxLabel = wx.StaticText(self.panel, -1, "nFmIdx", (0, 0))
        self.nFmDstLabel = wx.StaticText(self.panel, -1, "nFmDst", (0, 0))
        self.rMskLabel = wx.StaticText(self.panel, -1, "rMsk", (0, 0))
        self.tPosLabel = wx.StaticText(self.panel, -1, "timePos", (0, 0))
        self.tLfoFqLabel = wx.StaticText(self.panel, -1, "tLfoFq", (0, 0))
        self.tLfoAmtLabel = wx.StaticText(self.panel, -1, "tLfoAmt", (0, 0))
        self.waveXLabel = wx.StaticText(self.panel, -1, "waveX", (0, 0))
        self.waveYLabel = wx.StaticText(self.panel, -1, "waveY", (0, 0))
        self.chanMaskLabel = wx.StaticText(self.panel, -1, "chanMask", (0, 0))

        self.startLabel = wx.StaticText(self.panel, -1, "Start", (0, 0))
        self.time1Label = wx.StaticText(self.panel, -1, "Time1", (0, 0))
        self.curve1Label = wx.StaticText(self.panel, -1, "Curve1", (0, 0))
        self.middleLabel = wx.StaticText(self.panel, -1, "Middle", (0, 0))
        self.time2Label = wx.StaticText(self.panel, -1, "Time2", (0, 0))
        self.curve2Label = wx.StaticText(self.panel, -1, "Curve2", (0, 0))
        self.endLabel = wx.StaticText(self.panel, -1, "End", (0, 0))
        self.randDevLabel = wx.StaticText(self.panel, -1, "Random Deviation", (0, 0))
        self.rdStartLabel = wx.StaticText(self.panel, -1, "rdStart", (0, 0))
        self.rdTime1Label = wx.StaticText(self.panel, -1, "rdTime1", (0, 0))
        self.rdCurve1Label = wx.StaticText(self.panel, -1, "rdCurve1", (0, 0))
        self.rdMiddleLabel = wx.StaticText(self.panel, -1, "rdMiddle", (0, 0))
        self.rdTime2Label = wx.StaticText(self.panel, -1, "rdTime2", (0, 0))
        self.rdCurve2Label = wx.StaticText(self.panel, -1, "rdCurve2", (0, 0))
        self.rdEndLabel = wx.StaticText(self.panel, -1, "rdEnd", (0, 0))

        ## Add labels to sizer
        # 1st col, labels
        self.sizer1.Add(self.amp1Label       , (2,1))
        self.sizer1.Add(self.amp2Label       , (3,1))
        self.sizer1.Add(self.amp3Label       , (4,1))
        self.sizer1.Add(self.amp4Label       , (5,1))
        self.sizer1.Add(self.grRt1Label      , (6,1))
        self.sizer1.Add(self.grRt2Label      , (7,1))
        self.sizer1.Add(self.grRt3Label      , (8,1))
        self.sizer1.Add(self.grRt4Label      , (9,1))
        self.sizer1.Add(self.syncLabel       , (10,1))
        self.sizer1.Add(self.trsp1Label      , (11,1))
        self.sizer1.Add(self.trsp2Label      , (12,1))
        self.sizer1.Add(self.trsp3Label      , (13,1))
        self.sizer1.Add(self.trsp4Label      , (14,1))
        self.sizer1.Add(self.sweepToLabel    , (15,1))
        self.sizer1.Add(self.sweepTimeLabel  , (16,1))
        self.sizer1.Add(self.dur1Label       , (17,1))
        self.sizer1.Add(self.dur2Label       , (18,1))
        self.sizer1.Add(self.dur3Label       , (19,1))
        self.sizer1.Add(self.dur4Label       , (20,1))
        self.sizer1.Add(self.shapeLabel      , (21,1))
        self.sizer1.Add(self.atckLabel       , (22,1))
        self.sizer1.Add(self.gDistLabel      , (23,1))
        self.sizer1.Add(self.fmFqLabel       , (24,1))
        self.sizer1.Add(self.fmIdxLabel      , (25,1))
        self.sizer1.Add(self.nFmFqLabel      , (26,1))
        self.sizer1.Add(self.nFmIdxLabel     , (27,1))
        self.sizer1.Add(self.nFmDstLabel     , (28,1))
        self.sizer1.Add(self.rMskLabel       , (29,1))
        self.sizer1.Add(self.waveXLabel      , (30,1))
        self.sizer1.Add(self.waveYLabel      , (31,1))
        self.sizer1.Add(self.tLfoFqLabel     , (32,1))
        self.sizer1.Add(self.tLfoAmtLabel    , (33,1))
        self.sizer1.Add(self.tPosLabel       , (34,1))
        self.sizer1.Add(self.chanMaskLabel   , (35,1))

        # 2nd row, labels
        self.sizer1.Add(self.startLabel      , (1,2))
        self.sizer1.Add(self.time1Label      , (1,3))
        self.sizer1.Add(self.curve1Label     , (1,4))
        self.sizer1.Add(self.middleLabel     , (1,5))
        self.sizer1.Add(self.time2Label      , (1,6))
        self.sizer1.Add(self.curve2Label     , (1,7))
        self.sizer1.Add(self.endLabel        , (1,8))
        self.sizer1.Add(self.randDevLabel    , (1,9))
        self.sizer1.Add(self.rdStartLabel    , (1,10))
        self.sizer1.Add(self.rdTime1Label    , (1,11))
        self.sizer1.Add(self.rdCurve1Label   , (1,12))
        self.sizer1.Add(self.rdMiddleLabel   , (1,13))
        self.sizer1.Add(self.rdTime2Label    , (1,14))
        self.sizer1.Add(self.rdCurve2Label   , (1,15))
        self.sizer1.Add(self.rdEndLabel      , (1,16))


    def populateWidgetFullnames(self):
        for name in self.widgetBasenames:
            self.widgetFullnames.append(name+'Start')
            self.widgetFullnames.append(name+'Curve1')
            self.widgetFullnames.append(name+'Middle')
            self.widgetFullnames.append(name+'Curve2')
            self.widgetFullnames.append(name+'End')
            if name == 'v1Amp':
                self.widgetFullnames.append(name+'Time1')
                self.widgetFullnames.append(name+'Time2')

            if 'chanMask' not in name:
                name += 'Rdev'
                self.widgetFullnames.append(name+'Start')
                self.widgetFullnames.append(name+'Curve1')
                self.widgetFullnames.append(name+'Middle')
                self.widgetFullnames.append(name+'Curve2')
                self.widgetFullnames.append(name+'End')
                if name == 'v1AmpRdev':
                    self.widgetFullnames.append(name+'Time1')
                    self.widgetFullnames.append(name+'Time2')
            
    def setWidgetIDs(self, widgetlist):
        i = 0
        for widget in self.widgetFullnames:
            s = 'self.ID_%s = %i' %(widget, i)
            exec(s)
            i += 1
    
    def makeTextCtrlFromWidgetName(self, name):
        
        # define the strings that are to be executed as dynamic code
        s1 = 'self.%sCtrl = wx.TextCtrl(self.panel, self.ID_%s, "1.0", (0, 0), (40, 17))' %(name, name)
        if (name == 'chanMaskStart') or (name == 'chanMaskMiddle') or (name == 'chanMaskEnd'):
            s1 = 'self.%sCtrl = wx.TextCtrl(self.panel, self.ID_%s, "0,3,0,1,2,3", (0, 0), (140, 17))' %(name, name)

        s2 = 'wx.EVT_TEXT(self, self.ID_%s, self.onTextCtrl)' %name

        if name.find('Amp') > -1: colourGroup = 1
        elif name.find('GrRate') > -1: colourGroup = 2
        elif name.find('sync') > -1: colourGroup = 8
        elif name.find('Transp') > -1: colourGroup = 3
        elif name.find('sweep') > -1: colourGroup = 4
        elif name.find('GrDur') > -1: colourGroup = 5
        elif name.find('grShape') > -1: colourGroup = 6
        elif name.find('grAttack') > -1: colourGroup = 6
        elif name.find('grDist') > -1: colourGroup = 7
        elif name.find('fm') > -1: colourGroup = 8
        elif name.find('nois') > -1: colourGroup = 9
        elif name.find('randMask') > -1: colourGroup = 10
        elif name.find('timepos') > -1: colourGroup = 11
        elif name.find('tLfo') > -1: colourGroup = 11
        elif name.find('wave') > -1: colourGroup = 12
        elif name.find('chanMask') > -1: colourGroup = 13
        else: print name
        s3 = 'self.%sCtrl.SetBackgroundColour(ColourGroup%i)' %(name, colourGroup)
        
        # execute dynamic code
        exec(s1)
        exec(s2)
        exec(s3)
                
    def putWidgetInSizer(self, name):
        if name.find('Start') > -1 : hPosition = 2
        if name.find('Time1') > -1 : hPosition = 3
        if name.find('Curve1') > -1 : hPosition = 4
        if name.find('Middle') > -1 : hPosition = 5
        if name.find('Time2') > -1 : hPosition = 6
        if name.find('Curve2') > -1 : hPosition = 7
        if name.find('End') > -1 : hPosition = 8
        if name.find('Rdev') > -1 : hPosition += 8 # all Rdev controls in same layout to the right of other widgets

        if name.find('v1Amp') > -1 : vPosition       = 2
        if name.find('v2Amp') > -1 : vPosition       = 3
        if name.find('v3Amp') > -1 : vPosition       = 4
        if name.find('v4Amp') > -1 : vPosition       = 5
        if name.find('v1GrRate') > -1 : vPosition    = 6
        if name.find('v2GrRate') > -1 : vPosition    = 7
        if name.find('v3GrRate') > -1 : vPosition    = 8
        if name.find('v4GrRate') > -1 : vPosition    = 9
        if name.find('sync') > -1 : vPosition        = 10
        if name.find('v1Transp') > -1 : vPosition    = 11
        if name.find('v2Transp') > -1 : vPosition    = 12
        if name.find('v3Transp') > -1 : vPosition    = 13
        if name.find('v4Transp') > -1 : vPosition    = 14
        if name.find('sweepTo') > -1 : vPosition     = 15
        if name.find('sweepTime') > -1 : vPosition   = 16
        if name.find('v1GrDur') > -1 : vPosition     = 17
        if name.find('v2GrDur') > -1 : vPosition     = 18
        if name.find('v3GrDur') > -1 : vPosition     = 19
        if name.find('v4GrDur') > -1 : vPosition     = 20
        if name.find('grShape') > -1 : vPosition     = 21
        if name.find('grAttack') > -1 : vPosition    = 22
        if name.find('grDist') > -1 : vPosition      = 23
        if name.find('fmFreq') > -1 : vPosition      = 24
        if name.find('fmIndx') > -1 : vPosition      = 25
        if name.find('noisFmFreq') > -1 : vPosition  = 26
        if name.find('noisFmIndx') > -1 : vPosition  = 27
        if name.find('noisFmDistr') > -1 : vPosition = 28
        if name.find('randMask') > -1 : vPosition    = 29
        if name.find('waveX') > -1 : vPosition       = 30
        if name.find('waveY') > -1 : vPosition       = 31
        if name.find('tLfoFq') > -1 : vPosition      = 32
        if name.find('tLfoAmt') > -1 : vPosition     = 33
        if name.find('timepos') > -1 : vPosition     = 34
        if name.find('chanMask') > -1 : vPosition    = 35
                
        span = (1,1)
        if (name == 'chanMaskStart') or (name == 'chanMaskMiddle') or (name == 'chanMaskEnd'):
            span = (1,2)
        
        ctrl = compile('self.'+name+'Ctrl', 'string', 'eval')
        self.sizer1.Add(eval(ctrl), (vPosition, hPosition), span)
        
    def onTextCtrl(self, event):
        self.isEdited = True
        ID = event.GetEventObject().GetId()
        name = self.widgetFullnames[ID] 

        if (name.find('v1AmpTime') > -1) or (name.find('v1AmpRdevTime') > -1):
            if event.GetString() == '-': return
            time = float(event.GetString())
            # auto adjust time fractions for the two segments so that the total time will always be = 1.0
            if time >= 1.0:
                time = 0.99
            if time <= 0.0:
                time = 0.01
            self.eventCaller.cloudPlayer.parameters[name] = time
            # auto calculate time2
            time2 = 1.0 - time
            # update GUI and set the value for "the other" segment
            if name == 'v1AmpTime1':
                self.eventCaller.cloudPlayer.parameters['v1AmpTime2'] = time2
                self.eventCaller.guiCloudPreset.v1AmpTime2Ctrl.ChangeValue(str(time2))
            if name == 'v1AmpTime2':
                self.eventCaller.cloudPlayer.parameters['v1AmpTime1'] = time2
                self.eventCaller.guiCloudPreset.v1AmpTime1Ctrl.ChangeValue(str(time2))
            if name == 'v1AmpRdevTime1':
                self.eventCaller.cloudPlayer.parameters['v1AmpRdevTime2'] = time2
                self.eventCaller.guiCloudPreset.v1AmpRdevTime2Ctrl.ChangeValue(str(time2))
            if name == 'v1AmpRdevTime2':
                self.eventCaller.cloudPlayer.parameters['v1AmpRdevTime1'] = time2
                self.eventCaller.guiCloudPreset.v1AmpRdevTime1Ctrl.ChangeValue(str(time2))

        elif (name == 'chanMaskStart') or (name == 'chanMaskMiddle') or (name == 'chanMaskEnd'):
                chanMaskValues = util.makeListFromString(event.GetString())
                print 'onChanMask', chanMaskValues
                self.eventCaller.cloudPlayer.parameters[name] = chanMaskValues
            
        else:
            if event.GetString() == '-': return # if just a minus sign entered, allow more input before processing
            self.eventCaller.cloudPlayer.parameters[name] = float(event.GetString())

    def setWindowTitle(self, title):
        self.SetTitle(title)


def test():
    '''
    self test
    '''
    
    class Bogus:
        def __init__(self):
            self.parameters = {}
    eventCaller = Bogus()
    ctrl_app = wx.PySimpleApp()
    frame = CloudGUI(None, eventCaller)
    frame.Show(True)
    ctrl_app.MainLoop()

if __name__ == '__main__':
    test()
