#!/usr/bin/python
# -*- coding: latin-1 -*-
"""
cloudParameters
List of parameter base names for partikkel clouds.

The base names are used to form full parameter names.
e.g. v1Amp is used to form v1AmpStart, v1AmpMiddle, v1AmpEnd, v1AmpCurve1, v1AmpCurve2.
It is also used to form parameter names for random deviation,
this is the same as the above with the addition of 'Rdev' into the parameter name, e.g. v1AmpRdevStart
The above is done for all parameter basenames,
in addition, v1Amp also has Time1, Time2, RdevTime1, and RdevTime2 parameters (e.g. v1AmpRdevTime2)

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
"""

parameterBasenames = ['v1Amp',
                    'v2Amp',
                    'v3Amp',
                    'v4Amp',
                    'v1GrRate',
                    'v2GrRate',
                    'v3GrRate',
                    'v4GrRate',
                    'sync',
                    'v1Transp',
                    'v2Transp',
                    'v3Transp',
                    'v4Transp',
                    'sweepTo', 
                    'sweepTime', 
                    'v1GrDur', 
                    'v2GrDur', 
                    'v3GrDur', 
                    'v4GrDur', 
                    'grShape', 
                    'grAttack', 
                    'grDist',  
                    'fmFreq', 
                    'fmIndx', 
                    'noisFmFreq', 
                    'noisFmIndx', 
                    'noisFmDistr', 
                    'randMask', 
                    'timepos',  
                    'tLfoFq', 
                    'tLfoAmt', 
                    'waveX', 
                    'waveY', 
                    'chanMask']
                                
parameterFullnamesAndDefaults	= {
                	'v1AmpStart'         	: 1.0,
                	'v2AmpStart'         	: 1.0,
                	'v3AmpStart'         	: 1.0,
                	'v4AmpStart'         	: 1.0,
                	'v1GrRateStart'   	    : 1.0,
                	'v2GrRateStart'   	    : 1.0,
                	'v3GrRateStart'   	    : 1.0,
                	'v4GrRateStart'   	    : 1.0,
                	'syncStart'             : 0.0,
                	'v1TranspStart'   	    : 1.0,
                	'v2TranspStart'   	    : 1.0,
                	'v3TranspStart'   	    : 1.0,
                	'v4TranspStart'   	    : 1.0,
                    'sweepToStart'          : 1.0,
                    'sweepTimeStart'        : 1.0,
                	'v1GrDurStart'       	: 1.0,
                	'v2GrDurStart'       	: 1.0,
                	'v3GrDurStart'       	: 1.0,
                	'v4GrDurStart'       	: 1.0,
                    'grShapeStart'          : 1.0,
                	'grAttackStart'   	    : 1.0,
                	'grDistStart'        	: 1.0,
                	'fmFreqStart'        	: 1.0,
                	'fmIndxStart'        	: 1.0,
                	'noisFmFreqStart' 	    : 1.0,
                	'noisFmIndxStart' 	    : 1.0,
                	'noisFmDistrStart'   	: 1.0,
                	'randMaskStart'   	    : 1.0,
                	'timeposStart'       	: 1.0,
                    'tLfoFqStart'           : 1.0,
                    'tLfoAmtStart'          : 1.0,
                    'waveXStart'            : 1.0,
                    'waveYStart'            : 1.0,
                    'chanMaskStart'         : [0,3,0,1,2,3],
                	'v1AmpTime1'         	: 0.5,
                	'v1AmpCurve1'        	: 1.0,
                	'v2AmpCurve1'        	: 1.0,
                	'v3AmpCurve1'        	: 1.0,
                	'v4AmpCurve1'        	: 1.0,
                	'v1GrRateCurve1'  	    : 1.0,
                	'v2GrRateCurve1'  	    : 1.0,
                	'v3GrRateCurve1'  	    : 1.0,
                	'v4GrRateCurve1'  	    : 1.0,
                	'syncCurve1'            : 1.0,
                	'v1TranspCurve1'  	    : 1.0,
                	'v2TranspCurve1'  	    : 1.0,
                	'v3TranspCurve1'  	    : 1.0,
                	'v4TranspCurve1'  	    : 1.0,
                    'sweepToCurve1'         : 1.0,
                    'sweepTimeCurve1'       : 1.0,
                	'v1GrDurCurve1'   	    : 1.0,
                	'v2GrDurCurve1'   	    : 1.0,
                	'v3GrDurCurve1'   	    : 1.0,
                	'v4GrDurCurve1'   	    : 1.0,
                    'grShapeCurve1'         : 1.0,
                	'grAttackCurve1'  	    : 1.0,
                	'grDistCurve1'       	: 1.0,
                	'fmFreqCurve1'       	: 1.0,
                	'fmIndxCurve1'       	: 1.0,
                	'noisFmFreqCurve1'   	: 1.0,
                	'noisFmIndxCurve1'   	: 1.0,
                	'noisFmDistrCurve1'  	: 1.0,
                	'randMaskCurve1'  	    : 1.0,
                	'timeposCurve1'   	    : 1.0,
                    'tLfoFqCurve1'          : 1.0,
                    'tLfoAmtCurve1'         : 1.0,
                    'waveXCurve1'           : 1.0,
                    'waveYCurve1'           : 1.0,
                    'chanMaskCurve1'        : 1.0,
                	'v1AmpMiddle'       	: 1.0,
                	'v2AmpMiddle'        	: 1.0,
                	'v3AmpMiddle'        	: 1.0,
                	'v4AmpMiddle'        	: 1.0,
                	'v1GrRateMiddle'  	    : 1.0,
                	'v2GrRateMiddle'  	    : 1.0,
                	'v3GrRateMiddle'  	    : 1.0,
                	'v4GrRateMiddle'  	    : 1.0,
                	'syncMiddle'            : 0.0,
                	'v1TranspMiddle'  	    : 1.0,
                	'v2TranspMiddle'  	    : 1.0,
                	'v3TranspMiddle'  	    : 1.0,
                	'v4TranspMiddle'  	    : 1.0,
                    'sweepToMiddle'         : 1.0,
                    'sweepTimeMiddle'       : 1.0,
                	'v1GrDurMiddle'   	    : 1.0,
                	'v2GrDurMiddle'   	    : 1.0,
                	'v3GrDurMiddle'   	    : 1.0,
                	'v4GrDurMiddle'   	    : 1.0,
                    'grShapeMiddle'         : 1.0,
                	'grAttackMiddle'  	    : 1.0,
                	'grDistMiddle'       	: 1.0,
                	'fmFreqMiddle'       	: 1.0,
                	'fmIndxMiddle'       	: 1.0,
                	'noisFmFreqMiddle'   	: 1.0,
                	'noisFmIndxMiddle'   	: 1.0,
                	'noisFmDistrMiddle'  	: 1.0,
                	'randMaskMiddle'  	    : 1.0,
                	'timeposMiddle'   	    : 1.0,
                    'tLfoFqMiddle'          : 1.0,
                    'tLfoAmtMiddle'         : 1.0,
                    'waveXMiddle'           : 1.0,
                    'waveYMiddle'           : 1.0,
                    'chanMaskMiddle'        : [0,3,0,1,2,3],
                	'v1AmpTime2'         	: 0.5,
                	'v1AmpCurve2'        	: 1.0,
                	'v2AmpCurve2'        	: 1.0,
                	'v3AmpCurve2'        	: 1.0,
                	'v4AmpCurve2'        	: 1.0,
                	'v1GrRateCurve2'  	    : 1.0,
                	'v2GrRateCurve2'  	    : 1.0,
                	'v3GrRateCurve2'  	    : 1.0,
                	'v4GrRateCurve2'  	    : 1.0,
                	'syncCurve2'            : 0.0,
                	'v1TranspCurve2'  	    : 1.0,
                	'v2TranspCurve2'  	    : 1.0,
                	'v3TranspCurve2'  	    : 1.0,
                	'v4TranspCurve2'  	    : 1.0,
                    'sweepToCurve2'         : 1.0,
                    'sweepTimeCurve2'       : 1.0,
                    'v1GrDurCurve2'   	    : 1.0,
                    'v2GrDurCurve2'   	    : 1.0,
                	'v3GrDurCurve2'   	    : 1.0,
                    'v4GrDurCurve2'   	    : 1.0,
                    'grShapeCurve2'         : 1.0,
                    'grAttackCurve2'  	    : 1.0,
                    'grDistCurve2'       	: 1.0,
                    'fmFreqCurve2'       	: 1.0,
                    'fmIndxCurve2'       	: 1.0,
                    'noisFmFreqCurve2'   	: 1.0,
                    'noisFmIndxCurve2'   	: 1.0,
                    'noisFmDistrCurve2'  	: 1.0,
                    'randMaskCurve2'  	    : 1.0,
                    'timeposCurve2'   	    : 1.0,
                    'tLfoFqCurve2'          : 1.0,
                    'tLfoAmtCurve2'         : 1.0,
                    'waveXCurve2'           : 1.0,
                    'waveYCurve2'           : 1.0,
                    'chanMaskCurve2'        : 1.0,
                    'v1AmpEnd'           	: 1.0,
                    'v2AmpEnd'           	: 1.0,
                    'v3AmpEnd'           	: 1.0,
                    'v4AmpEnd'           	: 1.0,
                    'v1GrRateEnd'        	: 1.0,
                    'v2GrRateEnd'        	: 1.0,
                    'v3GrRateEnd'        	: 1.0,
                    'v4GrRateEnd'        	: 1.0,
                    'syncEnd'               : 0.0,
                    'v1TranspEnd'        	: 1.0,
                    'v2TranspEnd'        	: 1.0,
                    'v3TranspEnd'        	: 1.0,
                    'v4TranspEnd'        	: 1.0,
                    'sweepToEnd'            : 1.0,
                    'sweepTimeEnd'          : 1.0,
                    'v1GrDurEnd'         	: 1.0,
                    'v2GrDurEnd'         	: 1.0,
                    'v3GrDurEnd'         	: 1.0,
                    'v4GrDurEnd'         	: 1.0,
                    'grShapeEnd'            : 1.0,
                    'grAttackEnd'        	: 1.0,
                    'grDistEnd'          	: 1.0,
                    'fmFreqEnd'          	: 1.0,
                	'fmIndxEnd'          	: 1.0,
                	'noisFmFreqEnd'   	    : 1.0,
                	'noisFmIndxEnd'   	    : 1.0,
                	'noisFmDistrEnd'  	    : 1.0,
                	'randMaskEnd'        	: 1.0,
                	'timeposEnd'         	: 1.0,
                    'tLfoFqEnd'             : 1.0,
                    'tLfoAmtEnd'            : 1.0,
                    'waveXEnd'              : 1.0,
                    'waveYEnd'              : 1.0,
                    'chanMaskEnd'           : [0,3,0,1,2,3],
                    'v1AmpRdevStart'  	    : 1.0,
                    'v2AmpRdevStart'  	    : 1.0,
                    'v3AmpRdevStart'  	    : 1.0,
                    'v4AmpRdevStart'  	    : 1.0,
                    'v1GrRateRdevStart'  	: 1.0,
                    'v2GrRateRdevStart'  	: 1.0,
                    'v3GrRateRdevStart'  	: 1.0,
                    'v4GrRateRdevStart'  	: 1.0,
                	'syncRdevStart'         : 0.0,
                    'v1TranspRdevStart'  	: 1.0,
                    'v2TranspRdevStart'  	: 1.0,
                    'v3TranspRdevStart'  	: 1.0,
                    'v4TranspRdevStart'  	: 1.0,
                    'sweepToRdevStart'      : 1.0,
                    'sweepTimeRdevStart'    : 1.0,
                    'v1GrDurRdevStart'   	: 1.0,
                    'v2GrDurRdevStart'   	: 1.0,
                    'v3GrDurRdevStart'   	: 1.0,
                    'v4GrDurRdevStart'   	: 1.0,
                    'grShapeRdevStart'      : 1.0,
                    'grAttackRdevStart'  	: 1.0,
                    'grDistRdevStart' 	    : 1.0,
                    'fmFreqRdevStart' 	    : 1.0,
                    'fmIndxRdevStart' 	    : 1.0,
                    'noisFmFreqRdevStart'	: 1.0,
                    'noisFmIndxRdevStart'	: 1.0,
                    'noisFmDistrRdevStart'	: 1.0,
                    'randMaskRdevStart'  	: 1.0,
                    'timeposRdevStart'   	: 1.0,
                    'tLfoFqRdevStart'       : 1.0,
                    'tLfoAmtRdevStart'      : 1.0,
                    'waveXRdevStart'        : 1.0,
                    'waveYRdevStart'        : 1.0,
                    'v1AmpRdevTime1'  	    : 0.5,
                    'v1AmpRdevCurve1' 	    : 1.0,
                    'v2AmpRdevCurve1' 	    : 1.0,
                    'v3AmpRdevCurve1' 	    : 1.0,
                    'v4AmpRdevCurve1' 	    : 1.0,
                    'v1GrRateRdevCurve1' 	: 1.0,
                    'v2GrRateRdevCurve1' 	: 1.0,
                    'v3GrRateRdevCurve1' 	: 1.0,
                    'v4GrRateRdevCurve1' 	: 1.0,
                	'syncRdevCurve1'        : 0.0,
                    'v1TranspRdevCurve1' 	: 1.0,
                    'v2TranspRdevCurve1' 	: 1.0,
                    'v3TranspRdevCurve1' 	: 1.0,
                    'v4TranspRdevCurve1' 	: 1.0,
                    'sweepToRdevCurve1'     : 1.0,
                    'sweepTimeRdevCurve1'   : 1.0,
                    'v1GrDurRdevCurve1'  	: 1.0,
                    'v2GrDurRdevCurve1'  	: 1.0,
                    'v3GrDurRdevCurve1'  	: 1.0,
                    'v4GrDurRdevCurve1'  	: 1.0,
                    'grShapeRdevCurve1'     : 1.0,
                    'grAttackRdevCurve1' 	: 1.0,
                    'grDistRdevCurve1'   	: 1.0,
                    'fmFreqRdevCurve1'   	: 1.0,
                    'fmIndxRdevCurve1'   	: 1.0,
                    'noisFmFreqRdevCurve1'	: 1.0,
                    'noisFmIndxRdevCurve1'	: 1.0,
                    'noisFmDistrRdevCurve1'	: 1.0,
                    'randMaskRdevCurve1' 	: 1.0,
                    'timeposRdevCurve1'  	: 1.0,
                    'tLfoFqRdevCurve1'      : 1.0,
                    'tLfoAmtRdevCurve1'     : 1.0,
                    'waveXRdevCurve1'       : 1.0,
                    'waveYRdevCurve1'       : 1.0,
                    'v1AmpRdevMiddle' 	    : 1.0,
                    'v2AmpRdevMiddle' 	    : 1.0,
                    'v3AmpRdevMiddle' 	    : 1.0,
                    'v4AmpRdevMiddle' 	    : 1.0,
                    'v1GrRateRdevMiddle' 	: 1.0,
                    'v2GrRateRdevMiddle' 	: 1.0,
                    'v3GrRateRdevMiddle' 	: 1.0,
                    'v4GrRateRdevMiddle' 	: 1.0,
                	'syncRdevMiddle'        : 0.0,
                    'v1TranspRdevMiddle' 	: 1.0,
                    'v2TranspRdevMiddle' 	: 1.0,
                    'v3TranspRdevMiddle' 	: 1.0,
                    'v4TranspRdevMiddle' 	: 1.0,
                    'sweepToRdevMiddle'     : 1.0,
                    'sweepTimeRdevMiddle'   : 1.0,
                    'v1GrDurRdevMiddle'  	: 1.0,
                    'v2GrDurRdevMiddle'  	: 1.0,
                    'v3GrDurRdevMiddle'  	: 1.0,
                    'v4GrDurRdevMiddle'  	: 1.0,
                    'grShapeRdevMiddle'     : 1.0,
                    'grAttackRdevMiddle' 	: 1.0,
                    'grDistRdevMiddle'   	: 1.0,
                    'fmFreqRdevMiddle'   	: 1.0,
                    'fmIndxRdevMiddle'   	: 1.0,
                    'noisFmFreqRdevMiddle'	: 1.0,
                    'noisFmIndxRdevMiddle'	: 1.0,
                    'noisFmDistrRdevMiddle'	: 1.0,
                    'randMaskRdevMiddle' 	: 1.0,
                    'timeposRdevMiddle'  	: 1.0,
                    'tLfoFqRdevMiddle'      : 1.0,
                    'tLfoAmtRdevMiddle'     : 1.0,
                    'waveXRdevMiddle'       : 1.0,
                    'waveYRdevMiddle'       : 1.0,
                    'v1AmpRdevTime2'  	    : 0.5,
                    'v1AmpRdevCurve2' 	    : 1.0,
                    'v2AmpRdevCurve2' 	    : 1.0,
                    'v3AmpRdevCurve2' 	    : 1.0,
                    'v4AmpRdevCurve2' 	    : 1.0,
                    'v1GrRateRdevCurve2' 	: 1.0,
                    'v2GrRateRdevCurve2' 	: 1.0,
                    'v3GrRateRdevCurve2' 	: 1.0,
                    'v4GrRateRdevCurve2' 	: 1.0,
                	'syncRdevCurve2'        : 0.0,
                    'v1TranspRdevCurve2' 	: 1.0,
                    'v2TranspRdevCurve2' 	: 1.0,
                    'v3TranspRdevCurve2' 	: 1.0,
                    'v4TranspRdevCurve2' 	: 1.0,
                    'sweepToRdevCurve2'     : 1.0,
                    'sweepTimeRdevCurve2'   : 1.0,
                    'v1GrDurRdevCurve2'     : 1.0,
                    'v2GrDurRdevCurve2'     : 1.0,
                    'v3GrDurRdevCurve2'     : 1.0,
                    'v4GrDurRdevCurve2'     : 1.0,
                    'grShapeRdevCurve2'     : 1.0,
                    'grAttackRdevCurve2'    : 1.0,
                    'grDistRdevCurve2'      : 1.0,
                    'fmFreqRdevCurve2'      : 1.0,
                    'fmIndxRdevCurve2'      : 1.0,
                    'noisFmFreqRdevCurve2'	: 1.0,
                    'noisFmIndxRdevCurve2'	: 1.0,
                    'noisFmDistrRdevCurve2'	: 1.0,
                    'randMaskRdevCurve2'    : 1.0,
                    'timeposRdevCurve2'     : 1.0,
                    'tLfoFqRdevCurve2'      : 1.0,
                    'tLfoAmtRdevCurve2'     : 1.0,
                    'waveXRdevCurve2'       : 1.0,
                    'waveYRdevCurve2'       : 1.0,
                    'v1AmpRdevEnd'          : 1.0,
                    'v2AmpRdevEnd'          : 1.0,
                    'v3AmpRdevEnd'          : 1.0,
                    'v4AmpRdevEnd'          : 1.0,
                    'v1GrRateRdevEnd' 	    : 1.0,
                    'v2GrRateRdevEnd' 	    : 1.0,
                    'v3GrRateRdevEnd' 	    : 1.0,
                    'v4GrRateRdevEnd' 	    : 1.0,
                	'syncRdevEnd'        : 0.0,
                    'v1TranspRdevEnd' 	    : 1.0,
                    'v2TranspRdevEnd' 	    : 1.0,
                    'v3TranspRdevEnd' 	    : 1.0,
                    'v4TranspRdevEnd' 	    : 1.0,
                    'sweepToRdevEnd'        : 1.0,
                    'sweepTimeRdevEnd'      : 1.0,
                    'v1GrDurRdevEnd'  	    : 1.0,
                    'v2GrDurRdevEnd'  	    : 1.0,
                    'v3GrDurRdevEnd'  	    : 1.0,
                    'v4GrDurRdevEnd'  	    : 1.0,
                    'grShapeRdevEnd'        : 1.0,
                    'grAttackRdevEnd' 	    : 1.0,
                    'grDistRdevEnd'   	    : 1.0,
                    'fmFreqRdevEnd'   	    : 1.0,
                    'fmIndxRdevEnd'   	    : 1.0,
                    'noisFmFreqRdevEnd'     : 1.0,
                    'noisFmIndxRdevEnd'     : 1.0,
                    'noisFmDistrRdevEnd'    : 1.0,
                    'randMaskRdevEnd' 	    : 1.0, 
                    'timeposRdevEnd'  	    : 1.0,
                    'tLfoFqRdevEnd'         : 1.0,
                    'tLfoAmtRdevEnd'        : 1.0,
                    'waveXRdevEnd'          : 1.0,
                    'waveYRdevEnd'          : 1.0}
