#!/usr/bin/python
# -*- coding: latin-1 -*-

""" 
Setting up and running Csound.

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
@requires: csnd
"""

import threading
import wx
import csnd

class CsoundThread(threading.Thread):
    """ 
    Setting up and running Csound.
    """
    def __init__(self, csound, commandline):
        """
        Class constructor.
        
        @param self: The object pointer.
        @param csound: Pointer to the csound instance.
        @param commandLine: The csound commandline string.
        """
        threading.Thread.__init__(self)
        self.isRunning = True
        self.csound = csound
        self.commandline = commandline
        print "csound loaded"
        # Embed an orchestra in this script.
        self.csound.setOrchestra('''#include 'cloud.orc'
        ''')
        # And a score.
        self.csound.setScore('''#include 'cloud.sco'
        ''')
        # Command line
        self.csound.setCommand('''%s''' %self.commandline)
        # Export the orc and sco.
        self.csound.exportForPerformance()

    def run(self):
        """
        Run the csound k-rate loop.

        @param self: The object pointer.
        """
        # Start the performance.
        self.csound.compile()
        while(self.isRunning):
            self.csound.PerformKsmps()
            # Absolutely MUST yield to wxWindows or the app will freeze!
            wx.Yield()
        self.csound.Reset()
        self.csound.Cleanup()

    def stop(self):
        """
        stop the csound thread loop

        @param self: The object pointer.
        """
        print 'stopping csound'
        self.isRunning = False
    

