;   Copyright 2006 yvind Brandtsegg

;   All contributors can be found in credit.txt file

;   This file is part of ImproSculpt.

;   ImproSculpt is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.

;   ImproSculpt is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.

;   You should have received a copy of the GNU General Public License
;   along with ImproSculpt in the GeneralPublicLicense.txt file;
;   if not, write to the Free Software Foundation, 
;   Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#define PartikkelInstrCloud(A)#
; the $A argument distinguish between voices/instances of the instr
;***********
; basic parameters

;	isamplelen1	= 524288
;	kcpsSample1	= sr/isamplelen1

	iamp		= 0.6
	SvXAmp		= "v$AAmp"
	$readParm(kamp'SvXAmp)

	;kAmpXfade	chnget	"ParticleTypeXfade"			; crossfade between single particle generator and 4-voice particle cloud
	;kamp		= kamp * (1-kAmpXfade)

; soft sync
	id$A		= $A
	apulse$A	init 0
	aphase$A	init 0
	Ssync		= "sync"					; sync amount (pulse gravity)
	$readParm(ksyncGravity'Ssync)
	asyncPolarity	limit (int(aphase$A*2)*2)-1, -1, 1		; use the phase of partikkelsync instance n ($A) to find sync polarity for the sync signal.
									; If the phase of instance n is less than 0.5, we want to nudge it down when synchronizing,
									; and if the phase is > 0.5 we want to nudge it upwards.
	async$A		= apulse1*ksyncGravity*asyncPolarity		; prepare sync signal with polarity and strength
	async1		= 0						; do not sync voice 1 (it's the master)
	
; grain freq
	SvXGrRate	= "v$AGrRate"
	$readParm(kgrainfreq'SvXGrRate)
	agrainfreq	interp	kgrainfreq

	SgrDist		= "grDist"
	$readParmOffsetDev(kdistribution'SgrDist)		; periodic or stochastic distribution of grains, 0 = periodic
	
; grain transpose
	SvXTransp	= "v$ATransp"
	$readParm(kwavTransp'SvXTransp)
	kwavfreq	= kwavTransp				; grain wave frequency

	SsweepTo	= "sweepTo"
	$readParm(kwavFqCenter'SsweepTo)			; grain wave frequency gravity center (for sweeps)
	tablew		kwavFqCenter, 2, giWavfreqEndCloud	; modify table containing end freq for sweeps

	SsweepTime	= "sweepTime"				; sweep time is tha same as sweep shape
	$readParmOffsetDev(ksweepshape'SsweepTime)		; frequency sweep duration (as fraction of grain duration)

; grain shape
	ienv_attack	= giSigmoRise			; grain attack envelope table (must have extended guard point)
	ienv_decay	= giSigmoFall			; grain decay envelope table (must have extended guard point)
	SgrShape	= "grShape"
	$readParm(ksustain_amount'SgrShape)		; balance between enveloped time(attack+decay) and sustain level time, 0 = no time at sustain level

	SgrAttack	= "grAttack"
	$readParm(ka_d_ratio'SgrAttack)			; balance between attack time and decay time, 0 = zero attack time and full decay time

	SvXGrDur	= "v$AGrDur"
	$readParm(kGrainDur'SvXGrDur)			; grain duration, as multiple of grain frequency
	kduration	= (kGrainDur*1000)/kgrainfreq	; grain dur in milliseconds

; frequency modulation parameters and oscil
	SfmFreq	= "fmFreq"
	$readParm(kfmfq'SfmFreq)			; fm freq inside grain
	SfmIndx		= "fmIndx"
	$readParmOffsetDev(kfmindx'SfmIndx)			; fm index inside grain

	kfmindx		= (kfmindx * 0.1) + (kfmindx*kfmfq*0.00001)
;	kfmindx		= kfmindx * (kfmfq*0.0001)
	tablew		kfmindx, 2, giFmAmpsCloud
	ifmwave		= giSine
	awavfm		oscil	kfmindx, kfmfq, ifmwave	; audio input for frequency modulation inside grain
	awavFeed	chnget	"cloudAudioFeedback$A"
	awavFeed	= awavFeed * 10
	kFMselect	= (kfmfq < 1.0 ? 0 : 1)	; select oscillator or feedback modulation source, if frequency very low use feedback
	kFMselect	tonek	kFMselect, 1	; filter selector for smooth crossfade
	awavfm		= (awavfm * kFMselect) + (awavFeed * (1-kFMselect))
; noise FM
	SnoisFmIndx	= "noisFmIndx"
	$readParmOffsetDev(knFmIndx'SnoisFmIndx)			; noise fm index inside grain
	SnoisFmDistr	= "noisFmDistr"
	$readParm(knFmDist'SnoisFmDistr)		; noise fm distribution inside grain
	SnoisFmFreq	= "noisFmFreq"
	$readParm(knFmFreq'SnoisFmFreq)			; noise fm freq inside grain

	anoiseFM	rnd31	knFmIndx, knFmDist
	anoiseFM	butterlp anoiseFM, knFmFreq 
	awavfm		= awavfm + anoiseFM
;	awavfm		limit	awavfm, -1, 1

; trainlet parameters
	icosine		= giCosine		; for trainlets
	kTrainCps	chnget	"TrnCps"	; set cps equal to grain freq, for trainlets or single cycle waves
	ktrainamp	chnget	"TrnAmp"	; trainlet amp
	knumpartials	chnget	"TrnPart"	; number of partials in trainlet
	kchroma		chnget	"TrnChrom"	; chroma, falloff of partial amplitude towards sr/2

; masking
	igainmasks	= giGainMaskCloud			; gain masking table, amplitude for individual grains
	kgainmaskfrom	= 0;chnget	"not implemented"	; gain masking index start
	kgainmaskto	= 0;chnget	"not implemented"	; gain masking index end (loop point)
	tablew		kgainmaskfrom, 0, giGainMaskCloud
	tablew		kgainmaskto, 1, giGainMaskCloud

	ichannelmasks	= giChnMaskCloud			; channel masking table, pan for individual grains
	;kchannelmaskfrom jspline 2, 0.2, 1			; channel masking index start
	;kchannelmaskfrom = kchannelmaskfrom + 2
	;kchannelmaskto	jspline 2, 0.1, 1			; channel masking index end (loop point)
	;kchannelmaskto	= kchannelmaskto + 4
	;kchannelmaskfrom limit kchannelmaskfrom, 0, 6
	;kchannelmaskto	limit kchannelmaskto, 0, 6
	;tablew		kchannelmaskfrom, 0, giChnMaskCloud
	;tablew		kchannelmaskto, 1, giChnMaskCloud

	SrandMask	= "randMask"
	$readParmOffsetDev(krandommask'SrandMask)			; random grain masking


; waveforms
	SwaveX		= "waveX"
	SwaveY		= "waveY"
	$readParmOffsetDev(kJoyX'SwaveX)
	$readParmOffsetDev(kJoyY'SwaveY)
	kwgain1		limit sqrt((1-kJoyX)*(1-kJoyY)), 0, 1
	kwgain2		limit sqrt(kJoyX*(1-kJoyY)), 0, 1
	kwgain3		limit sqrt((1-kJoyX)*kJoyY), 0, 1
	kwgain4		limit sqrt(kJoyX*kJoyY), 0, 1
;	printks		"joy values %f %f %f %f %n", 1, kwgain1, kwgain2, kwgain3, kwgain4
	kwaveform1	chnget "particleWave1"		; audio waveform inside each grain
	kwaveform2	chnget "particleWave2"		; waveform 2
	kwaveform3	chnget "particleWave3"		; waveform 3
	kwaveform4	chnget "particleWave4"		; waveform 4

	kwavegain1	= kwgain1 * (1-ktrainamp)	; gain (mix) for waveform 1, crossfade with trainlet
	tablew		kwavegain1, 2, giWavAmpsCloud
	kwavegain2	= kwgain2 * (1-ktrainamp)	; gain (mix) for waveform 2
	tablew		kwavegain2, 3, giWavAmpsCloud
	kwavegain3	= kwgain3 * (1-ktrainamp)	; gain (mix) for waveform 3
	tablew		kwavegain3, 4, giWavAmpsCloud
	kwavegain4	= kwgain4 * (1-ktrainamp)	; gain (mix) for waveform 4
	tablew		kwavegain4, 5, giWavAmpsCloud
	tablew		ktrainamp, 6, giWavAmpsCloud	; gain (mix) for trainlets

; time
	Stimepos	= "timepos"
	$readParm(ksamplepos'Stimepos)			; start position for reading grain waveform
	StLfoFq	= "tLfoFq"
	$readParm(kTimeLfoFreq'StLfoFq)			; LFO freq for timepos modulation
	StLfoAmt	= "tLfoAmt"
	$readParmOffsetDev(kTimeLfoAmt'StLfoAmt)			; LFO amount for timepos modulation
	kTimeLfo	oscil	kTimeLfoAmt, kTimeLfoFreq, giTri
	ksamplepos	= ksamplepos + kTimeLfo
	ksamplepos	tonek	ksamplepos, 3
	asamplepos	interp	ksamplepos
	asamplepos1	= asamplepos
	asamplepos2	= asamplepos
	asamplepos3	= asamplepos
	asamplepos4	= asamplepos
	imax_grains	= 100			; max number of grains per k-period

;***********


;* nye parameter
idisttab	= giProbDistCloud		; random distribution for grain masking
iwaveamptab	= giWavAmpsCloud		; amp per wave per grain
kenv2amt	= 0.5			; additional enveloping per grain (e.g. for fof synthesis)
ienv2tab	= giSigmoFall
iwavfreqstarttab = giWavfreqStartCloud
iwavfreqendtab	= giWavfreqEndCloud
ifmamptab	= giFmAmpsCloud
kfmenv		init giSigmoFall

;*************************************
; calculation of timepoint, transpose, and waveform selection when using live sampled segments
if kwaveform1 < 10000 goto noSegment1$A
	; find audio segment start and end position
	kSegment1	= kwaveform1 - 10000
	kStart1		table kSegment1, giLiveSamplAttackTable
	kEnd1		table kSegment1+1, giLiveSamplAttackTable

	; adjust samplepos
	kLength1	= (kEnd1-kStart1)/giLiveSamplTableLen
	asamplepos1 	= (ksamplepos*kLength1) + (kStart1/giLiveSamplTableLen)

	; adjust kwavekey
	kwavekey1	= sr/giLiveSamplTableLen

	; set live sampling table as waveform
	kwaveform1a	= giLiveSamplAudioTable
	goto passWaveform1$A

noSegment1$A:
kLen1		tableng	kwaveform1
kwavekey1	= sr/kLen1
kwaveform1a	= kwaveform1

passWaveform1$A:

if kwaveform2 < 10000 goto noSegment2$A
	; find audio segment start and end position
	kSegment2	= kwaveform2 - 10000
	kStart2		table kSegment2, giLiveSamplAttackTable
	kEnd2		table kSegment2+1, giLiveSamplAttackTable

	; adjust samplepos
	kLength2	= (kEnd2-kStart2)/giLiveSamplTableLen
	asamplepos2 	= (ksamplepos*kLength2) + (kStart2/giLiveSamplTableLen)

	; adjust kwavekey
	kwavekey2	= sr/giLiveSamplTableLen

	; set live sampling table as waveform
	kwaveform2a	= giLiveSamplAudioTable
	goto passWaveform2$A

noSegment2$A:
kLen2		tableng	kwaveform2
kwavekey2	= sr/kLen2
kwaveform2a	= kwaveform2

passWaveform2$A:


if kwaveform3 < 10000 goto noSegment3$A
	; find audio segment start and end position
	kSegment3	= kwaveform3 - 10000
	kStart3		table kSegment3, giLiveSamplAttackTable
	kEnd3		table kSegment3+1, giLiveSamplAttackTable

	; adjust samplepos
	kLength3	= (kEnd3-kStart3)/giLiveSamplTableLen
	asamplepos3 	= (ksamplepos*kLength3) + (kStart3/giLiveSamplTableLen)

	; adjust kwavekey
	kwavekey3	= sr/giLiveSamplTableLen

	; set live sampling table as waveform
	kwaveform3a	= giLiveSamplAudioTable
	goto passWaveform3$A

noSegment3$A:
kLen3		tableng	kwaveform3
kwavekey3	= sr/kLen3
kwaveform3a	= kwaveform3

passWaveform3$A:


if kwaveform4 < 10000 goto noSegment4$A
	; find audio segment start and end position
	kSegment4	= kwaveform4 - 10000
	kStart4		table kSegment4, giLiveSamplAttackTable
	kEnd4		table kSegment4+1, giLiveSamplAttackTable

	; adjust samplepos
	kLength4	= (kEnd4-kStart4)/giLiveSamplTableLen
	asamplepos4 	= (ksamplepos*kLength4) + (kStart4/giLiveSamplTableLen)

	; adjust kwavekey
	kwavekey4	= sr/giLiveSamplTableLen

	; set live sampling table as waveform
	kwaveform4a	= giLiveSamplAudioTable
	goto passWaveform4$A

noSegment4$A:
kLen4		tableng	kwaveform4
kwavekey4	= sr/kLen4
kwaveform4a	= kwaveform4

passWaveform4$A:


a1, a2, a3, a4 partikkel agrainfreq, kdistribution, idisttab, async$A, kenv2amt, ienv2tab, \
               ienv_attack, ienv_decay, ksustain_amount, ka_d_ratio, kduration, kamp, igainmasks, \
               kwavfreq, ksweepshape, iwavfreqstarttab, iwavfreqendtab, awavfm, \
               ifmamptab, kfmenv, icosine, kTrainCps, knumpartials, \
               kchroma, ichannelmasks, krandommask, kwaveform1, kwaveform2, kwaveform3, kwaveform4, \
               iwaveamptab, asamplepos1, asamplepos2, asamplepos3, asamplepos4, \
               kwavekey1, kwavekey2, kwavekey3, kwavekey4, imax_grains, id$A

apulse$A,aphase$A	partikkelsync	id$A

amono	= a1+a2+a3+a4
chnset	amono, "cloudAudioFeedback$A"		; audio feedback for fm

a1	= a1 * iamp
a2	= a2 * iamp
a3	= a3 * iamp
a4	= a4 * iamp

chnmix	a1, "partikkelOut1"
chnmix	a2, "partikkelOut2"
chnmix	a3, "partikkelOut3"
chnmix	a4, "partikkelOut4"
;outs a1 + a3, a2 + a4

#

; (end of macro definition)